//
//  PaymentEngineMiddleware.h
//  Middleware-iOSLibrary
//
//  Created by PaymentEngine on 7/30/15.
//  Copyright (c) 2015 PaymentEngine. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "PaymentEngineTransactionResponse.h"

@protocol PaymentEngineMiddlewareDelegate <NSObject>

@required

/*!
    @brief Delegate method for startTransaction. 
    @param transResponse - Contains all the transaction response info from the gateway
 */
-(void)transactionComplete :(PaymentEngineTransactionResponse *)transResponse;
@optional

/*!
    @brief Delegate method for device connected. The method gets called when the device is connected
 */
-(void)deviceConnected;

/*!
  @brief Delegate method for device disconnected. The method gets called when the device is disconnected
 */
-(void)deviceDisconnected;

/*!
  @brief Call this delegate method when the receipt is returned from the gateway
  @param receiptValue - Contains all the necessary data for a receipt
 */
-(void)returnReceiptCompleted :(NSString *)receiptValue;

/*!
    @brief Delegate method, it will get call when we start to update the terminal config
 */
-(void)startUpdatingTerminal;

/*!
  @brief Delegate method, it will get call once we finish updating all the terminal config
 */
-(void)updateTerminalConfigComplete;

/*!
    @briefThis is a call back method when the signature is captured
    @param transResponse - Contains all the transaction information returned from the server
 */
-(void)captureSignatureComplete :(PaymentEngineTransactionResponse *)transResponse;

/*!
    @brief Delegate method for getMerchantCapabilities
    @param merchantCapDict - Dictionary that holds information about the merchant and the features they support
 */
-(void)getMerchantCapabilitiesComplete: (NSDictionary *)merchantCapDict;

/*!
 @brief Delegate method for readDeviceInfo
 @param deviceInfoDict - Dictionary that holds information about the merchant and the features they support
 */
-(void) readDeviceInfoComplete :(NSDictionary *)deviceInfoDict;

@end

@interface PaymentEngineMiddleware : NSObject

+ (PaymentEngineMiddleware *) getInstance;

@property(nonatomic, strong) id<PaymentEngineMiddlewareDelegate> delegate;

//------------------------------Device Methods------------------------------

/*!
    @brief This method must be called before starting any transaction. Use it to connect to a device.
    @param deviceName The name of the device such as icmp or castle
    @param setDelegate Sets to self
 
 */
-(void)setDevice :(NSString *)deviceName :(id)setDelegate;

/*! @brief Returns true if the device is online, false otherwise */
-(BOOL)isDeviceOnline;

/*! @brief Disconnect the terminal from the device */
-(void)disconnectDevice;

/*! @brief Connects the terminal to the device */
-(void)connectDevice;

/*! @brief (Not supported on MP200) This method takes the icmp device to go online */
-(void)goOnline;

/*! @brief (Not supported on MP200) This method takes the icmp device to go offline */
-(void)goOffline;

/*! @brief Deprecated in v2.3.5 - Returns all the information related to the device such as firmware, device model and etc... */
-(NSDictionary *)getDeviceInfo;

/*!
        @brief Sends a command to the device to retrieve a list of device information such as battery level, firmware, and etc..
        @delegate Callback method - readDeviceInfoComplete :(NSDictionary *)deviceInfo
 */
-(void)readDeviceInfo;

/*! @brief Cancels a transaction that is still in progress */
-(void)cancelTransaction;

//------------------------------Gateway Methods------------------------------

/*!
        @brief Use the method whenever a transaction is ready to be process
        @param holdTransInfo - A NSmutableDicitonary that holds all the transaction
 */
-(void)startTransaction :(NSMutableDictionary *)holdTransInfo;

/*!
        @brief Use the method to capture a signature
        @param holdCaptureInfo - Contains all the transaction info and signature image in base64 encoded format
 
 */
-(void)captureSignature:(NSMutableDictionary *)holdCaptureInfo;

/*!
        @brief Use the method to obtain receipt from gateway
        @param receiptDict - Contains all the necessary information for a receipt
 */
-(void)getReceipt :(NSDictionary *)receiptDict;

/*!
        @brief Use the method to obtain merchant's capability
        @param autoUpdateConfigs - Setting it to true will automatically updates the merchant capaiblity in the device if the device is connected
 */
-(void)getMerchantCapabilities :(BOOL)autoUpdateConfigs;


@end
